@@ -6,7 +6,8 @@ from .statuses import ParcelCarrierSize, ParcelLockerSize, ParcelDeliveryType, P
     ParcelServiceName, ParcelStatus
 from .exceptions import NoParcelError, UnidentifiedParcelError, ParcelTypeError, NotAuthenticatedError, ReAuthenticationError, \
     PhoneNumberError, SmsCodeError, RefreshTokenError, UnidentifiedAPIError, UserLocationError, \
-    UnidentifiedError, NotFoundError, UnauthorizedError, SingleParamError
+    UnidentifiedError, NotFoundError, UnauthorizedError, SingleParamError, MissingParamsError
 from .endpoints import login, send_sms_code, confirm_sms_code, refresh_token, parcels, parcel, collect, \
-    compartment_open, compartment_status, terminate_collect_session, friends, shared, sent, returns, parcel_prices, \
-    tickets, logout, multi
+    compartment_open, compartment_status, terminate_collect_session, friendship, shared, sent, returns, parcel_prices, \
+    tickets, logout, multi, validate_friendship, accept_friendship
+from .friends import Friend
